/* printctl1.c - convert ASCII printer text files to 133 byte lines     */
/*             - with IBM carriage control character in 1st byte        */
/* by Owen Townsend, www.uvsoftware.ca, Aug31/2011                      */
/* This program is free software: you may redistribute it and/or modify */
/* it under the terms of GPLv3 (GNU General Public License version 3)   */
/* see free download info at www.uvsoftware.ca/freestuff.htm            */

/* UV Software markets Vancouver Utilities www.uvsoftware.ca/uvprices.htm  */
/* Also see free program 'uvhd' file investigation utility, great for files*/
/* with packed/binary fields &/or without LineFeeds. You will need uvhd to */
/* inspect the 133 byte output of printctl1 which do not have LineFeeds    */

/* This program converts LFs & FFs to 1st byte control codes as follows:  */
/* LineFeed LF=x'0A', FormFeed FF=x'0C', Carriage Return CR=x'0D'         */
/* counts LFs/FFs prior after data strings (non LF/FF/CR) & prior to next */
/* 1+ FFs --> '1' FormFeed         */
/* 1  LF   --> ' ' space 1 before  */
/* 2  LFs  --> '0' space 2 before  */
/* 3+ LFs  --> '-' space 3 before  */
/* 0  LFs  --> ' ' space 0 before  */

/* cc src/printctl1.c -o bin/printctl1       <-- compile */
/* ===================================                   */
/* bin/printctl1 tf/printctl_test1 tmp/test1 <-- execute */
/* =========================================             */

#include <stdio.h>
#include <string.h>
char f1n[64];            /* input filename                    */
FILE *f1p;               /* file pointer from fopen           */
char f1rec[256];         /* input record area                 */
int cc;                  /* current fgetc return char/err/EOF */
int ii,oo;               /* indices for storing/writing chars */

char f2n[64];            /* output filename                   */
FILE *f2p;               /* file pointer from fopen           */
int f2stat;              /* current fwrite return status      */
int f2recs;              /* record count                      */

char ctlchar;            /* control character                 */
                         /* ' '=sp1,'0'=sp2,'-'=sp3,'+'=sp0   */
                         /* '1'=formfeed                      */
char line2[256];         /* output line with ctlcode 1st byte */
                         /* max data 133                      */

int lfcnt;       /* count LFs between ASCII lines */
int ffcnt;       /* count FFs between ASCII lines */

/*eject*/
int main(int argc, char **argv)
{
if (argc != 3)
  { printf("usage: printctl1 infilename outfilename \n");
    return(1);
  }

strcpy(f1n,argv[1]);     /* store fieldnames from cmnd line  */
strcpy(f2n,argv[2]);

if ((f1p = fopen(f1n,"rb")) == (FILE*)0)  /* <-- NOTE "rb" Binary spcfd */
  { printf("cant open input file %s \n",f1n);
    return(3);
  }
if ((f2p = fopen(f2n,"wb")) == (FILE*)0)
  { printf("cant open output file %s \n",f2n);
    return(4);
  }

/*eject*/
/* begin main loop to read input characters & write output records */
while (1)
  { /* begin each line, init counters & data collection area */
    lfcnt=0; ffcnt=0; 
    memset(f1rec,'\0',256);  /* init input read area to nulls */
    memset(line2,' ',256);   /* init output line area to blanks */

    /* 1st get LFs,FFs,CRs after prior line & before next line */
    /* - count LFs & FFs to deterine ctlchar for folwng data line */
    /* begin loop to get bytes until non LF/FF/CR */
    while(1)
      { cc = fgetc(f1p);          /* get next character */
        if (cc < 0)               /* test EOF ? */
          { break; 
          }
        if (cc == '\r')
          { continue;
          } 
        if (cc == '\n')
          { lfcnt++;
            continue;
          }
        if (cc == '\f')
          { ffcnt++;
            continue;
          }
        else
          { break;
          }
      }

    /* end LFs,FFs,CRs (1st data char reached) */
    if (cc < 0)                  /* test EOF reached above */
      { break;
      }
    ungetc(cc,f1p);              /* unget 1st data for consistent logic */
    /* collect data chars until 1st folwng LF,FF,CR */
    ii = 0;                      /* init index to data collect area */
    while(1)
      { cc = fgetc(f1p);          /* get next character */
        if (cc < 0)               /* test EOF ? */
          { break; 
          }
        if ((cc == '\r') || (cc == '\n') || (cc == '\f'))
          { break;
          } 
        else
          { f1rec[ii++] = cc;    /* store current character */
          }
      }

/*eject*/
    /* end data chars, 1st LF,FF,CR reached */
    if (cc < 0)                  /* test EOF reached above */
      { break;
      }
    ungetc(cc,f1p);              /* unget 1st data for consistent logic */

    /* format ctl code depending on counts LFs, FFs */
    if (ffcnt)
      { ctlchar = '1';
      }
    else
      { if (lfcnt == 1)
          { ctlchar = ' ';
          }
        else if (lfcnt == 0)
          { ctlchar = '+';
          }
        else if (lfcnt == 1)
          { ctlchar = ' ';
          }
        else if (lfcnt == 2)
          { ctlchar = '0';
          }
        else 
          { ctlchar = '-';        /* default space 3 if LFs >= 3 */
          }
      }

    /* create data record - 1st byte control code + 132 bytes data/blanks */
    line2[0] = ctlchar;
    memcpy(line2+1,f1rec,ii);     /* copy data bytes collected */
    f2stat = fwrite(line2,1,133,f2p);
    if (f2stat < 0)
      { printf("ERROR, writing output file %s \n",f2n);
        return(5);
      }
    f2recs++;                             /* inc count for line seq#  */
 }

/* close files, display EOF msg with record count & exit */
fclose(f1p);
fclose(f2p);
printf("EOF: copied file %s to %s, output-records=%d \n",f1n,f2n,f2recs);
return(0);
}

